# LAB 09 - Impact analysis in R – the effects of the COVID-19 pandemic to the oil industry ----
#
# In this lab we are analyzing the impact of the COVID-19 pandemic to the oil
# industry. We will use oil price and production data, and we will "forecast"
# them for a 2020 in a scenario without the pandemic. Then we compare to what
# actually happened. We will use the library Modeltime for the forecasting.


# 1.0 - Libraries ----

# API
library(Quandl)

# Time Series
library(lubridate)
library(timetk)
library(tsibble)

# Visualization
library(plotly)

# Core
library(tidyverse)
library(tidyquant)
library(data.table)

# Model
library(tidymodels)
library(modeltime)
library(modeltime.ensemble)


# 2.0 - Loading and Visualizing the Data ----

# My Quandl key. To obtain a Quandl key, register using
# the link https://www.quandl.com/
# Quandl.api_key("Add your key here")
Quandl.api_key(Sys.getenv("QUANDL_API_KEY"))

# Setting API key for the EIA WTI crude oil price
eia_wti <- "EIA/PET_RWTC_D"

# Downloading data
wti_prices <- Quandl(
  code = eia_wti, 
  start_date = "2005-01-01", 
  end_date = today(),
  order = "asc"
) %>%
  as_tibble()

wti_prices %>% plot_time_series(
  Date,
  Value,
  .smooth = F,
  .interactive = T, 
  .title = "WTI Crude Oil Price (USD)",
  .x_lab = "Date",
  .y_lab = "Price (USD)",
  .color_lab = "Year",
  .plotly_slider = T
)

# Now the production data
eia_production <- "EIA/PET_WCRFPUS2_W"

# Production data is the daily average per week, and the unit is
# thousand barrels per day
wti_production <- Quandl(
  code = eia_production,
  start_date = "2005-01-01",
  end_date = today(),
  order = "asc"
) %>% 
  as_tibble()

wti_production %>% plot_time_series(
  Date,
  Value,
  .smooth = F,
  .interactive = T, 
  .title = "WTI Crude Oil Production",
  .x_lab = "Date",
  .y_lab = "Production (kbbl)",
  .color_lab = "Year",
  .plotly_slider = T
)

# 3.0 - Changing Time Unit ----

set_time_unit <- function(data, time_unit = "month", .fun = mean) {
  
  output <- data %>%
    mutate(Date = floor_date(Date, unit = time_unit)) %>%
    
    group_by(Date) %>%
    summarize(Value = .fun(Value)) %>%
    ungroup()
  
  return(output)
}

# Prices
wti_price_month <- wti_prices %>% 
  set_time_unit(time_unit = "month",
                .fun = mean)

wti_price_month %>% plot_time_series(
  Date,
  Value,
  .smooth = F,
  .interactive = T, 
  .title = "WTI Crude Oil Price (USD)",
  .x_lab = "Date",
  .y_lab = "Price (USD)",
  .color_lab = "Year",
  .plotly_slider = T
)

# Production
wti_production_month <- wti_production %>% 
  set_time_unit(time_unit = "month",
                .fun = mean)

wti_production_month %>% plot_time_series(
  Date,
  Value,
  .smooth = F,
  .interactive = T, 
  .title = "WTI Crude Oil Production",
  .x_lab = "Date",
  .y_lab = "Production (kbbl)",
  .color_lab = "Year",
  .plotly_slider = T
)


# 4.0 - Subset the Data and Split into Train/Test ----

# Price
splits_price <- wti_price_month %>%
  # subset(Date < "2020-01-01") %>% 
  time_series_split(assess = "11 months", cumulative = TRUE)

splits_price %>%
  tk_time_series_cv_plan() %>%
  plot_time_series_cv_plan(
    Date, 
    Value, 
    .interactive = T, 
    .title = "WTI Oil Price (USD)",
    .x_lab = "Date",
    .y_lab = "WTI Oil Price (USD)",
    .plotly_slider = T
  )

# Production
splits_production <- wti_production_month %>%
  # subset(Date < "2020-01-01") %>% 
  time_series_split(assess = "11 months", cumulative = TRUE)

splits_production %>%
  tk_time_series_cv_plan() %>%
  plot_time_series_cv_plan(
    Date, 
    Value, 
    .interactive = T, 
    .title = "WTI Oil Production",
    .x_lab = "Date",
    .y_lab = "WTI Oil Production (kbbl)",
    .plotly_slider = T
  )

# 5.0 - Models Calibration ----

## 5.1 - ARIMA ----
# Prices
arima_prices <- arima_reg() %>%
  set_engine(engine = "auto_arima") %>%
  fit(Value ~ Date, training(splits_price))

arima_prices

# Production
arima_production <- arima_reg() %>%
  set_engine(engine = "auto_arima") %>%
  fit(Value ~ Date, training(splits_production))

arima_production

## 5.2 - ARIMA + XGBoost ----
# Prices
barima_prices <- arima_boost() %>% 
  set_engine("auto_arima_xgboost") %>% 
  fit(Value ~ Date + as.numeric(Date) + factor(month(x = Date), ordered = F),
      data = training(splits_price))

barima_prices

# Production
barima_production <- arima_boost() %>% 
  set_engine("auto_arima_xgboost") %>% 
  fit(Value ~ Date + as.numeric(Date) + factor(month(x = Date), ordered = F),
      data = training(splits_production))

barima_production

## 5.3 - Facebook Prophet ----
# Prices
prophet_prices <- prophet_reg(
  changepoint_range = 1
) %>% 
  set_engine("prophet") %>% 
  fit(Value ~ Date, training(splits_price))

prophet_prices

# Prices
prophet_production <- prophet_reg(
  changepoint_range = 1
) %>% 
  set_engine("prophet") %>% 
  fit(Value ~ Date, training(splits_production))

prophet_production

## 5.4 - Prophet + XGBoost ----
# Prices
bprophet_prices <- prophet_boost(
  changepoint_range = 1
) %>% 
  set_engine("prophet_xgboost") %>% 
  fit(Value ~ Date + as.numeric(Date) + factor(month(x = Date), ordered = F),
      data = training(splits_price))

bprophet_prices

# Production
bprophet_production <- prophet_boost(
  changepoint_range = 1
) %>% 
  set_engine("prophet_xgboost") %>% 
  fit(Value ~ Date + as.numeric(Date) + factor(month(x = Date), ordered = F),
      data = training(splits_production))

bprophet_production

## 5.5 - Ensemble ----
# Prices
models_table <- modeltime_table(
  arima_prices,
  barima_prices,
  prophet_prices,
  bprophet_prices
)

ensemble_prices <- models_table %>%
  ensemble_average(type = "mean")

calibration_table_prices <- modeltime_table(
  arima_prices,
  barima_prices,
  prophet_prices,
  bprophet_prices,
  ensemble_prices
) %>% 
  modeltime_calibrate(testing(splits_price))

# Production
models_table <- modeltime_table(
  arima_production,
  barima_production,
  prophet_production,
  bprophet_production
)

ensemble_production <- models_table %>%
  ensemble_average(type = "mean")

calibration_table_production <- modeltime_table(
  arima_production,
  barima_production,
  prophet_production,
  bprophet_production,
  ensemble_production
) %>% 
  modeltime_calibrate(testing(splits_production))


# 6.0 - Forecasting ----
# Prices
forecast_prices <- calibration_table_prices %>%
  modeltime_forecast(
    new_data = testing(splits_price), 
    actual_data = wti_price_month) 

forecast_prices %>%
  plot_modeltime_forecast(.interactive = TRUE)

# Production
forecast_production <- calibration_table_production %>%
  modeltime_forecast(
    new_data = testing(splits_production), 
    actual_data = wti_production_month)

forecast_production %>%
  plot_modeltime_forecast(.interactive = TRUE)


# 7.0 - Impact Analysis ----

models <- unique(forecast_prices$.model_desc)

impact <- forecast_prices[forecast_prices$.model_desc == models[1],] %>% 
  subset(select = c(.index, .value)) %>% 
  rename(Date = .index, Prices.T = .value) %>% 
  subset(Date > "2019-12-31") %>% 
  append(
    forecast_prices[forecast_prices$.model_desc == models[6],] %>% 
      subset(select = c(.value)) %>% 
      rename(Prices.P = .value)
  ) %>% 
  append(
    forecast_production[forecast_production$.model_desc == models[1],] %>% 
      subset(.index > "2019-12-31") %>% 
      subset(select = c(.value)) %>% 
      rename(Production.T = .value)
  ) %>% 
  append(
    forecast_production[forecast_production$.model_desc == models[6],] %>% 
      subset(select = c(.value)) %>% 
      rename(Production.P = .value)
  ) %>% 
  as_tibble() %>% 
  mutate(
    revenue.T = Prices.T * Production.T * 1000,
    revenue.P = Prices.P * Production.P * 1000,
    days.month = days_in_month(Date),
    loss.average = revenue.P - revenue.T,
    loss.total = loss.average * days.month,
    loss.cumulative = cumsum(loss.total)
  )

# Plotting
impact %>% 
  plot_ly() %>% 
  add_lines(
    x = ~Date,
    y = ~loss.cumulative,
    name = "Cumulative Loss",
    fill = 'tozeroy',
    alpha = 0.3, 
    line = list(width = 1),
    hoverinfo = 'text',
    text = ~paste("</br>Month: ", months(Date),
                  "</br>Loss: ", round(loss.cumulative/(1.e9) ,1), "B USD")
  ) %>% 
  add_lines(
    x = ~Date,
    y = ~loss.total,
    name = "Monthly Loss",
    fill = 'tozeroy',
    alpha = 0.3, 
    line = list(width = 1),
    hoverinfo = 'text',
    text = ~paste("</br>Month: ", months(Date),
                  "</br>Loss: ", round(loss.total/(1.e9) ,1), "B USD")
  ) %>% 
  layout(
    title = "WTI Crude Oil Revenue Loss in 2020",
    yaxis = list(title = "Revenue Loss (USD)"),
    legend = list(x = 0.01, 
                  y = 0.99,
                  title=list(text='<b> Legend </b>'))
  )







